/*****************************************************************************
 *                                                                           *
 *  Websphere MQ Classes for Java Message Service                            *
 *  Sample Pub/Sub Application                                               *
 *                                                                           *
 *  Licensed Materials - Property of IBM Corp.                               *
 *  5648-C60 (c) Copyright IBM Corp. 1999                                    *
 *                                                                           *
 *****************************************************************************
 *                                                                           *
 *  Usage:                                                                   *
 *      runjms JMSPubSub { -pub | -sub }                                     *
 *                                                                           *
 *  where:                                                                   *
 *      -pub = run program using publisher personality                       *
 *      -sub = run program using subscriber personality                      *
 *                                                                           *
 *      Typically, two instances of this program will be run: one using the  *
 *      publisher personality, the other using the subscriber personality.   *
 *                                                                           *
 *****************************************************************************
 *                                                                           *
 *  IMPORTANT NOTE:                                                          *
 *                                                                           *
 *  For this sample to run correctly, it requires two administered objects   *
 *  to be placed in a JNDI namespace pointed to by the INIT_URL and          *
 *  CTX_FACTORY variables, which will need editing to suit your particular   *
 *  installation.                                                            *
 *                                                                           *
 *  These objects can be created using the following JMS Administration      *
 *  Tool commands:                                                           *
 *                                                                           *
 *      DEFINE TCF(PubSub.TCF)                                               *
 *                                                                           *
 *      DEFINE T(PubSub.T) TOPIC(MQJMS/Samples/PubSub)                       *
 *                                                                           *
 *****************************************************************************/

import javax.jms.*;                   // JMS classes

import java.io.*;                     // Standard Java imports
import java.util.*;

import javax.naming.*;                // JNDI imports
import javax.naming.directory.*;

public class JMSPubSub
{
    /*************************************************************************
     *  main - program entry point                                           *
     *************************************************************************/
    public static void main( String args[] ) throws Exception
    {
        // First check to see which 'personality' the program is being run
        // in: publisher (-pub) or subscriber (-sub). Store this information
        // in the 'mode' variable for later
        char mode = ' ';

        if( args.length>0 ) {
            if( args[0].toUpperCase().equals("-PUB") )
                mode = 'p';
            else if( args[0].toUpperCase().equals("-SUB") )
                mode = 's';
        }

        // Check that one of the modes has been specified. If it hasn't, then
        // report this and exit
        if( mode==' ' ) {
            System.out.println( "You must specify -pub or -sub" );
            System.exit(-1);
        }

        // Create the keyboard reader, so that we can get user input
        BufferedReader in = new BufferedReader( new InputStreamReader(
                                                                System.in ));

        // Attempt to connect to the JNDI namespace in which the administered
        // objects we need exist. The two strings that follow may need
        // alteration to suit your particular installation
        String CTX_FACTORY = "com.sun.jndi.ldap.LdapCtxFactory";
        String INIT_URL    = "ldap://polaris/cn=PubSub,o=ibm_us,c=us";

        Hashtable env = new Hashtable();
        env.put( Context.INITIAL_CONTEXT_FACTORY, CTX_FACTORY );
        env.put( Context.PROVIDER_URL, INIT_URL );
        env.put( Context.REFERRAL, "throw" );
        Context ctx = new InitialDirContext( env );

        // Obtain the TopicConnectionFactory from JNDI
        TopicConnectionFactory tcf =
                       (TopicConnectionFactory)ctx.lookup( "cn=PubSub.TCF" );

        // Create a topic connection
        TopicConnection tConn = tcf.createTopicConnection();

        // Don't forget to start the connection
        tConn.start();

        // Create a topic session from the connection
        TopicSession tSess = tConn.createTopicSession(
                                            false, Session.AUTO_ACKNOWLEDGE );

        // Obtain the Topic from JNDI
        Topic t = (Topic)ctx.lookup( "cn=PubSub.T" );

        // Now that we have the Topic, we can start the main loop of the
        // program, remembering to create either the TopicPublisher or the
        // TopicSubscriber on the first run round the loop
        TopicPublisher  tPub = null;
        TopicSubscriber tSub = null;
        TextMessage     msg  = null;

        boolean carryOn = true;
        while( carryOn )
        {
            if( mode=='s' )
            {
                // Here is the subscriber code. Create the TopicSubscriber
                // if we haven't already done so
                if( tSub==null )
                    tSub = tSess.createSubscriber(t);

                // Print out the options available to the subscriber (here
                // we provide three flavors of the receive call), and read
                // back the selection
                System.out.print( "\n[R]eceiveBlock, Receive[N]oWait, "+
                                  "Receive[5]Secs, [Q]uit? " );
                String input = in.readLine();
                char c = ' ';

                if( input.length()>0 )
                    c = input.charAt(0);

                switch( c ) {
                    case 'r':
                    case 'R':
                        // This performs a receive which will not return
                        // until a message arrives on the subscriber queue
                        msg = (TextMessage)tSub.receive();
                        System.out.println(msg);
                        break;

                    case 'n':
                    case 'N':
                        // This performs a receive which returns immediately,
                        // without a message if one wasn't available
                        msg = (TextMessage)tSub.receiveNoWait();
                        if( msg==null ) System.out.println("No message!");
                        else            System.out.println(msg);
                        break;

                    case '5':
                        // This performs a receieve which waits 5 seconds
                        // before returning, possibly without a message
                        msg = (TextMessage)tSub.receive(5000);
                        if( msg==null ) System.out.println("No message!");
                        else            System.out.println(msg);
                        break;

                    case 'q':
                    case 'Q':
                        // Break out of the main program loop
                        carryOn = false;
                        break;
                }
            }
            else if( mode=='p' )
            {
                // And here is the publisher code. Create the TopicPublisher
                // if we haven't already done so
                if( tPub==null )
                    tPub = tSess.createPublisher(t);

                // Print out the options available to the publisher, and
                // read back the selection
                System.out.print( "\n[P]ublish message, [Q]uit? " );
                String input = in.readLine();
                char c = ' ';

                if( input.length()>0 )
                    c = input.charAt(0);

                switch( c ) {
                    case 'p':
                    case 'P':
                        // To publish a message, create an empty one...
                        msg = tSess.createTextMessage();

                        // ...set the message content...
                        System.out.print( "Text to publish : " );
                        String payload = in.readLine();
                        msg.setText(payload);

                        // ...and publish it
                        tPub.publish(msg);
                        break;

                    case 'q':
                    case 'Q':
                        // Break out of the main program loop
                        carryOn = false;
                        break;
                }
            }
        }

        // We must remember to call the close() method on all the objects we
        // have used, to ensure a clean tidy-up of all resources held
        if( mode=='p' )
            tPub.close();
        else if( mode=='s' )
            tSub.close();

        tSess.close();
        tConn.close();
    }
}
